#!/bin/bash

# Author:  Panagiotis Velissariou <pvelissariou@fsu.edu>
#                                 <velissariou.1@osu.edu>
# Version: 1.0
#
# Version - 1.0 Mon Aug 31 2015


# The script name and the directory where this script is located
scrNAME="$( basename ${0} )"
scrDIR="$( dirname ${0} )"
pushd ${scrDIR} > /dev/null 2>&1
  scrDIR="$( pwd )"
popd > /dev/null 2>&1
cd ${scrDIR}


#------------------------------------------------------------
# UTILITY FUNCTIONS
#
[[ ! :$PATH: == *:".":* ]] && export PATH="${PATH}:."

if [ -f functions_run ]; then
  . functions_run
else
  echo " ### ERROR: in ${scrNAME}"
  echo "     Cannot locate the file: functions_run"
  echo "     Exiting now ..."
  echo
  exit 1
fi
#------------------------------------------------------------
#DATA_PROD=coamps0.2a
#DATA_PROD=nogaps0.5c
#DATA_PROD=nogaps0.5a
DATA_PROD=navgem0.5a

# CLEANUP > 0 means that the resulting temporary idl files
# will be removed after this script finishes its calculations
CLEANUP=1


#------------------------------------------------------------
##### The name of the filename of main model input file.
#     This file should exist and be readable by the current user.
MODEL_INP="${MODEL_INP:-blkdat.input-tmpl}"

##### Call ParseArgs to get all supplied options to the script.
ParseArgs "${@}"
#------------------------------------------------------------

theMODEL="roms"
theDOM="${DOM_ROMS}"


#------------------------------------------------------------
# BEG:: Calculations

############################################################
##### Misc. script variables
ListIdlFiles=
ListBatchFiles=

############################################################
##### Create the IDL files for theMODEL
if [ ${NO_INI} -le 0 ]; then
  IdlFilesIni ${theMODEL} ${theDOM}
  ListIdlFiles="${ListIdlFiles} ${listIDL}"
  ListBatchFiles="${ListBatchFiles} ${listBATCH}"
  unset listIDL listBATCH
fi
if [ ${NO_BRY} -le 0 ]; then
  IdlFilesBry ${theMODEL} ${theDOM}
  ListIdlFiles="${ListIdlFiles} ${listIDL}"
  ListBatchFiles="${ListBatchFiles} ${listBATCH}"
  unset listIDL listBATCH
fi

############################################################
##### Check for the required IDL files
ListIdlFiles="$( strTrim "${ListIdlFiles}" 2 )"
ListBatchFiles="$( strTrim "${ListBatchFiles}" 2 )"
if [ "X${ListIdlFiles}" = "X" ]; then
  procError "no idl files were defined"
fi


############################################################
##### Run the parallel program
echo "        Creating the \"$(echo ${theMODEL} | tr [a-z] [A-Z])\" boundary and initial conditions files ..."

pushd ${IdlDir} >/dev/null
  GPAR_JOBLOG="${LogDir}/${scrNAME%%.*}-status.log"
  GPAR_RUNLOG="${LogDir}/${scrNAME%%.*}-run.log"

  GPAR_OPTS="${GPAR_OPTS_GLB} ${GPAR_OPTS_SSH} ${GPAR_OPTS_TIME}"
  GPAR_OPTS="${GPAR_OPTS} --joblog ${GPAR_JOBLOG} ${GPAR_OPTS_RESUME}"
  GPAR_OPTS="${GPAR_OPTS} --wd ${IdlDir} -j0"

  # Remove any old log files
  [ -f ${GPAR_RUNLOG} ] && rm -f ${GPAR_RUNLOG}
  [ -f ${GPAR_JOBLOG} ] && rm -f ${GPAR_JOBLOG}

  # We cannot run parallel in the background, somehow remote jobs are not
  # killed properly when a failure occurs and subsequently parallel does not exit
  ${GPAR} ${GPAR_OPTS} ${IDL_CMD} {} ::: $(cat ${ListBatchFiles}) > ${GPAR_RUNLOG} 2>&1
  FAILURE_STATUS=$?

  if [ ${FAILURE_STATUS} -eq 0 -a ${CLEANUP} -ge 1 ]; then
    echo "        Cleaning all INI/BRY related temporary files ..."
    for ilist in ${ListIdlFiles} ${ListBatchFiles}
    do
      for ifile in $(cat ${ilist})
      do
        [ -f "${ifile}" ] && rm -f "${ifile}"
      done
      [ -f "${ilist}" ] && rm -f "${ilist}"
    done
  fi
popd >/dev/null

if [ ${FAILURE_STATUS} -eq 0 ]; then
  imod="${theMODEL}"
  dom_str="_d`get2DString ${theDOM}`"

  pushd ${BryDir} >/dev/null
    for ityp in clim bry
    do
      for imo in {1..12}
      do
        regex="[1-9][0-9][0-9][0-9]"
        mo_str="`get2DString ${imo}`"
        files=( $( find . -mindepth 1 -maxdepth 1 -type f \
                         -iname "${imod}${ityp}${dom_str}_${regex}-${mo_str}*.nc" \
                         -exec basename {} \; | sort -u ) )
        if [ ${#files[@]} -ne 0 ]; then
          this_date="$( echo "${files[0]%%.*}" | sed -e "s@${imod}${ityp}${dom_str}_@@g" | sed -e 's/_/ /g' )"
          files="$( strTrim "${files[*]}" 2 )"
          file_stamp="$( date -d "${this_date}" "+%Y-%m-%d_%H:00:00" )"
          file_out="${imod}${ityp}${dom_str}_${file_stamp}.nc"
          ncrcat -h ${files} ${file_out}.tmp
          if [ $? -eq 0 ]; then
            rm -f ${files}
            mv -f ${file_out}.tmp ${file_out}
            #link_stamp="$( date -d "${this_date}" "+%Y${mo_str}" )"
            #linkFILE "${file_out}" "${imod}${ityp}${dom_str}-${link_stamp}.nc"
          fi
        fi
      done
    done
  popd >/dev/null

#  pushd ${IniDir} >/dev/null
#    for ityp in init
#    do
#      file_stamp="$( date -d "${SimBeg}" "+%Y-%m-%d_%H:00:00" )"
#      file_out="${imod}${ityp}${dom_str}_${file_stamp}.nc"
#      if [ -f "${file_out}" ]; then
#        link_stamp="$( date -d "${SimBeg}" "+%Y%m" )"
#        linkFILE "${file_out}" "${imod}${ityp}${dom_str}-${link_stamp}.nc"
#      fi
#    done
  popd >/dev/null
fi

for ilog in ${GPAR_JOBLOG} ${GPAR_RUNLOG}
do
  log_file="${ilog}"
  stripESCFILE "${log_file}"
done

# END:: Calculations
#------------------------------------------------------------

exit ${FAILURE_STATUS:-0}
